package me.mrletsplay.fixtreasuremaps.mixin;

import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import com.mojang.datafixers.util.Pair;

import me.mrletsplay.fixtreasuremaps.FixTreasureMapsMod;
import net.minecraft.entity.passive.DolphinEntity;
import net.minecraft.loot.function.ExplorationMapLootFunction;
import net.minecraft.registry.entry.RegistryEntry;
import net.minecraft.registry.entry.RegistryEntryList;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.math.BlockPos;
import net.minecraft.village.TradeOffers;
import net.minecraft.world.gen.chunk.ChunkGenerator;
import net.minecraft.world.gen.structure.Structure;

@Mixin(ChunkGenerator.class)
public class ChunkGeneratorMixin {

	@Inject(at = @At("HEAD"), method = "locateStructure(Lnet/minecraft/server/world/ServerWorld;Lnet/minecraft/registry/entry/RegistryEntryList;Lnet/minecraft/util/math/BlockPos;IZ)Lcom/mojang/datafixers/util/Pair;", cancellable = true)
	private void locateStructure(ServerWorld world, RegistryEntryList<Structure> structures, BlockPos center, int radius, boolean skipReferencedStructures, CallbackInfoReturnable<Pair<BlockPos, RegistryEntry<Structure>>> info) {
		StackTraceElement[] st = Thread.currentThread().getStackTrace();
		if(st.length < 5) return;

		// Expected stack trace:
		//
		// getStackTrace()
		// this method
		// ChunkGenerator#locateStructure
		// ServerWorld#locateStructure
		// method that called locateStructure

		String invokingClassName = st[4].getClassName();
		FixTreasureMapsMod.LOGGER.debug("locateStructure called by " + invokingClassName);
		if(invokingClassName.equals(ExplorationMapLootFunction.class.getName())
			|| invokingClassName.equals(TradeOffers.SellMapFactory.class.getName())
			|| invokingClassName.startsWith(DolphinEntity.class.getName())) { // DolphinEntity$LeadToNearbyTreasureGoal
			FixTreasureMapsMod.LOGGER.debug("Using optimized treasure map generation");
			info.setReturnValue(((ChunkGeneratorAccessor) this).invokeLocateStructure(world, structures, center, 20, skipReferencedStructures));
		}else {
			FixTreasureMapsMod.LOGGER.debug("NOT using optimized treasure map generation");
		}
	}

}
